using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.XR;
using System;

namespace Unity.XR.Pimax
{

    public class PvrEyeTracking
    {

        /// <summary>
        /// Get the PosMatrix of the head.
        /// </summary>
        /// <param name="matrix">Receive the Matrix4x4 returned</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetHeadPosMatrix(out Matrix4x4 matrix)
        {
            matrix = Matrix4x4.identity;

            if (!EyeTracking())
                return false;

            InputDevice device = default(InputDevice);
            if (!GetEyeTrackingDevice(out device))
                return false;

            Vector3 headPos = Vector3.zero;
            if (!device.TryGetFeatureValue(CommonUsages.devicePosition, out headPos))
            {
                PvrLog.LogError("GetHeadPosMatrix Pos Failed");
                return false;
            }

            Quaternion headRot = Quaternion.identity;
            if (!device.TryGetFeatureValue(CommonUsages.deviceRotation, out headRot))
            {
                PvrLog.LogError("GetHeadPosMatrix Rot Failed");
                return false;
            }

            matrix = Matrix4x4.TRS(headPos, headRot, Vector3.one);
            return true;
        }

        /// <summary>
        /// Gets the input device for eye tracking.
        /// </summary>
        /// <param name="device">Receive the InputDevice returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        static bool GetEyeTrackingDevice(out InputDevice device)
        {
            device = default(InputDevice);

            if (!EyeTracking())
                return false;

            List<InputDevice> devices = new List<InputDevice>();
            InputDevices.GetDevicesWithCharacteristics(InputDeviceCharacteristics.EyeTracking | InputDeviceCharacteristics.HeadMounted, devices);
            if (devices.Count == 0)
            {
                PvrLog.LogError("GetEyeTrackingDevice Failed, get none eyeTracking device");
                return false;
            }
            device = devices[0];
            if (!device.isValid)
                PvrLog.LogError("GetEyeTrackingDevice Failed device is not Valid");
            return device.isValid;
        }

        /// <summary>
        /// Get the position of the combined gaze point of left and right eye.
        /// </summary>
        /// <param name="point">Receive the vector 3 returned</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetCombineEyeGazePoint(out Vector3 point)
        {
            point = Vector3.zero;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetCombineEyeGazePoint pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            point = FloatArrayToVecter3(pose.combinedEyeGazePoint);
            return true;
        }

        /// <summary>
        /// Get the position of the left eye gaze point.
        /// </summary>
        /// <param name="point">Receive the vector 3 returned</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetLeftEyeGazePoint(out Vector3 point)
        {
            point = Vector3.zero;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetLeftEyeGazePoint pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            point = FloatArrayToVecter3(pose.leftEyeGazePoint);
            return true;
        }

        /// <summary>
        /// Get the position of the right eye gaze point.
        /// </summary>
        /// <param name="point">Receive the vector 3 returned</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetRightEyeGazePoint(out Vector3 point)
        {
            point = Vector3.zero;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetRightEyeGazePoint pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            point = FloatArrayToVecter3(pose.rightEyeGazePoint);
            return true;
        }

        /// <summary>
        /// Get the direction of the combined gaze point of left and right eye.
        /// </summary>
        /// <param name="vector">Receive the vector 3 returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetCombinedEyeGazeVector(out Vector3 vector)
        {
            vector = Vector3.zero;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetCombinedEyeGazeVector pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            vector = FloatArrayToVecter3(pose.combinedEyeGazeVector);
            return true;
        }

        /// <summary>
        /// Get the direction of the left eye gaze point.
        /// </summary>
        /// <param name="vector">Receive the vector 3 returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetLeftEyeGazeVector(out Vector3 vector)
        {
            vector = Vector3.zero;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetLeftEyeGazeVector pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            vector = FloatArrayToVecter3(pose.leftEyeGazeVector);
            return true;
        }

        /// <summary>
        /// Get the direction of the right gaze point.
        /// </summary>
        /// <param name="vector">Receive the vector 3 returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetRightEyeGazeVector(out Vector3 vector)
        {
            vector = Vector3.zero;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetRightEyeGazeVector pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            vector = FloatArrayToVecter3(pose.rightEyeGazeVector);
            return true;
        }

        /// <summary>
        /// Get the openness of the left eye.
        /// </summary>
        /// <param name="openness">Receive the float openness returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetLeftEyeOpenness(out float openness)
        {
            openness = 0f;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetLeftEyeOpenness pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            openness = pose.leftEyeOpenness;
            return true;
        }

        /// <summary>
        /// Get the openness of the right eye.
        /// </summary>
        /// <param name="openness">Receive the float openness returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetRightEyeOpenness(out float openness)
        {
            openness = 0f;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetRightEyeOpenness pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            openness = pose.rightEyeOpenness;
            return true;
        }

        /// <summary>
        /// Gets the pose status of the left eye.
        /// </summary>
        /// <param name="status">Receive the int status returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetleftEyePoseStatus(out Int32 status)
        {
            status = 0;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetleftEyePoseStatus pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            status = pose.leftEyePoseStatus;
            return true;
        }

        /// <summary>
        /// Gets the pose status of the right eye.
        /// </summary>
        /// <param name="status">Receive the int status returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetRightEyePoseStatus(out Int32 status)
        {
            status = 0;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetRightEyePoseStatus pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            status = pose.rightEyePoseStatus;
            return true;
        }

        /// <summary>
        /// Get the status of the combined of left and right eyes.
        /// </summary>
        /// <param name="status">Receive the int status returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetCombinedEyePoseStatus(out Int32 status)
        {
            status = 0;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetCombinedEyePoseStatus pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            status = pose.combinedEyePoseStatus;
            return true;
        }

        /// <summary>
        /// Get the position of the left eyes in a coordinate system.
        /// </summary>
        /// <param name="position">Receive the position returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetLeftEyePositionGuide(out Vector3 position)
        {
            position = Vector3.zero;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetLeftEyePositionGuide pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            position = FloatArrayToVecter3(pose.leftEyePositionGuide);
            return true;
        }

        /// <summary>
        /// Get the position of the right eyes in a coordinate system.
        /// </summary>
        /// <param name="position">Receive the position returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetRightEyePositionGuide(out Vector3 position)
        {
            position = Vector3.zero;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetRightEyePositionGuide pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            position = FloatArrayToVecter3(pose.rightEyePositionGuide);
            return true;
        }

        /// <summary>
        /// Gets the gaze direction for foveation rendering.
        /// </summary>
        /// <param name="direction">Receive the vector 3 returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetFoveatedGazeDirection(out Vector3 direction)
        {
            direction = Vector3.zero;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetFoveatedGazeDirection pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            direction = FloatArrayToVecter3(pose.foveatedEyeGazeVector);
            return true;
        }

        /// <summary>
        /// Gets the current Tracking state for foveated gaze direction.
        /// </summary>
        /// <param name="state">Receive the int state returned.</param>
        /// <returns>
        /// True - succeed, false - failed
        /// </returns>
        public static bool GetFoveatedGazeTrackingState(out Int32 state)
        {
            state = 0;

            if (!EyeTracking())
                return false;

            sxrEyePoseState pose = default(sxrEyePoseState);
            SxrResult result = PvrPlugin.pvr_GetEyePose(ref pose);
            if (result != SxrResult.SXR_ERROR_NONE)
            {
                PvrLog.LogError(string.Format("GetFoveatedGazeTrackingState pvr_GetEyePose Failed code: {0}", result));
                return false;
            }

            state = pose.foveatedEyeGazeState;
            return true;
        }



        //-----------------------Tools-----------------------


        public static Vector3 FloatArrayToVecter3(float[] FloatArray)
        {
            return new Vector3(FloatArray[0], FloatArray[1], FloatArray[2]);
        }

        static bool EyeTracking()
        {
#if UNITY_ANDROID
            if (!PvrSettings.instance.eyeTracking)
            {
                return false;
            }
#endif
            return true;
        }
    }
}
